// UrsADC
//
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-adc.html
// Created: 2017-01-08
//
// Version 1.0
// - Basis-Version


#ifndef _URS_ADC_h
#define _URS_ADC_h

#if defined(ARDUINO) && ARDUINO >= 100
	#include "arduino.h"
#else
	#include "WProgram.h"
#endif

class UrsAdcClass
{ protected:
   uint8_t  Samples = 6;           // Anzahl der Messungen, die zusammengefasst werden sollen (min. 3, max. 16).
   uint16_t Hysteresis = 3;        // Hysterese zur Messwertglttung
   uint16_t LastCommitedValue = 0; // Basis fr die Hysterese
   uint16_t LastValue = 0;         // Letzter bereinigter Wert ohne Hysterese
   uint16_t TempValues[16];        // Zwischenspeicher fr die max. 16 Messwerte   
   uint16_t MaxVal = 0;            // Max. Wert der max. 16 ADC-Werte
   uint16_t MinVal = 0xFFFF;       // Min. Wert
   uint8_t  MaxInd;                // Index des max. Wertes im Feld temp[]
   uint8_t  MinInd;                // Index des min. Wertes
   bool     SpecialZeroTreatment = true; // Sonderbehandlung fr 0-Werte aktivieren
  public:
    UrsAdcClass();  

   // Mittelt ADC-Werte zur besseren Stabilitt der ADC-Readings.
   // Es werden Samples (Voreinstellung = 6) aufeinanderfolgende Messwerte aufgenommen.
   // Der jeweils hchste und niedrigste wird verworfen.
   // Der Rckgabewert ist der Mittelwert aus den Samples-2 verbliebenen Werten (4 bei Voreinstellung).
   // Wenn eine Verbindung zum WLAN besteht, dauert ein Messzyklus bei der Voreinstellung (Samples = 6)
   // im Schnitt knapp 600s. Gelegentlich werden auch knapp 700s bentigt, wahrscheinlich weil
   // Netzwerk-Operationen dazwischen kommen.
	 uint16_t read();
   
   // Legt die Anzahl der Messwerte fest, aus denen das Ergebnis gemittelt wird.
   // Max. ist 16, min. ist 3, Voreinstellung ist 6.
   void setSampleCount(uint8_t Value) {if (Value > 16) Value = 16; if (Value < 3) Value = 3;  Samples = Value;};
   
   // Ruft die Anzahl der eingestellten Messungen zur Festlegung des bereinigten Wertes ab.
   uint8_t getSampleCount() {return Samples;};

   // Legt die Hysterese zur Glttung der Messwerte fest.
   // Max. ist 64, min. ist 0, Voreinstellung ist 3.
   void setHysteresis(uint16_t Value) { if (Value > 64) Value = 64; Hysteresis = Value; };

   // Ruft die Hysterese zur Glttung der Messwerte ab.
   uint8_t getHysteresis() { return Hysteresis; };

   // Legt fest, ob die Sonderbehandlung fr 0-Werte erfolgen soll.
   void setSpecialZeroTreatment(bool Value) { SpecialZeroTreatment = Value; };

   // Ruft ab, ob die Sonderbehandlung fr 0-Werte erfolgt.
   uint8_t getSpecialZeroTreatment() { return SpecialZeroTreatment; };

   // Ruft den letzten teilweise aufbereiteten Messwert ab.
   // (Ausreier eliminiert und Mittelwert gebildet, aber keine Anwendung der Hysterese 
   // und keine Sonderbehandlung der Null)
   uint16_t getLastValue() {return LastValue;}
};

extern UrsAdcClass UrsAdc;

#endif

